@@ -6,6 +6,7 @@ use warnings;
 use Carp;
 use List::MoreUtils qw/minmax uniq/;
 use File::Basename;
+use File::Copy 'mv';
 use feature 'say';
 
 
@@ -139,9 +140,9 @@ sub new {
    pulsStart =>'1 2 40 280',
    respStart =>'1 2 20 2',
    sliceOrder => 'alt+z',
+   MRDiscardNum => 0, # how many MR volues to discard
    VERB=>0,
    trustIdx=>'none',
-
   );
   # use defaults when we didn't set anything
   for my $k (keys %defaults) {
@@ -159,7 +160,7 @@ after intializing p, provide a file name
 
 =head3 input file format
 
- 1 2 40 280 ...
+ 1 2 40 280 ... [long space delimn. list of measurements, maybe 5000 for trigger?]
  ECG  Freq Per: 0 0
  PULS Freq Per: 74 807
  RESP Freq Per: 20 2860
@@ -221,6 +222,10 @@ sub readPhysio {
 
 
    # get start and end from settings
+   if(!$settings{"LogStart${timetyp}Time"} or !$settings{"LogStop${timetyp}Time"}) {
+     croak "Cannot find Log(Start|Stop)*Time in $fname";
+   }
+
    $self->{physStart} = $settings{"LogStart${timetyp}Time"}/1000;
    $self->{physEnd}   = $settings{"LogStop${timetyp}Time"}/1000;
 
@@ -236,8 +241,9 @@ sub readPhysio {
 
 
    say "file is $self->{ptype} with $#{$self->{measures}} samples, " ,
-       "$self->{physStart}s - $self->{physEnd}s, ",
-       "sample rate adjusted to $self->{PhRate}"
+       "$self->{physStart}s - $self->{physEnd}s (",
+       sprintf("%.2f",($self->{physEnd}-$self->{physStart})/60),
+       "min), sample rate adjusted to $self->{PhRate}"
      if $self->{VERB};
 
    # does the time match the sample rate and number of samples
@@ -302,11 +308,23 @@ sub readMRdir {
  # @v is an element for each dcm (line of output)
  my @v=`$dcmcmd` or croak "could not run $dcmcmd";
 
+
  # make each line an array
  # so we have an array of arrays
  # v[0] is all info on first dicom
  # v[0][0] is the first dicom's file name
  @v= map { [split / /] } @v;
+ 
+ # Sort by acq time (just in case file names are out of order)
+ # only needed to ensure trunction takes out the right dicoms
+ my $ati=getidx('AcqTime');
+ @v = sort {$a->[$ati] <=> $b->[$ati]} @v;
+
+ # Truncate dicom list if we have volumes to discard
+ # for some multi-band protocols, the first dicom is junk
+ # for older protocols, we have to manually discard volumes
+ #  -- eitherway we dont want them
+ @v=@v[($self->{MRDiscardNum})..$#v] if($self->{MRDiscardNum});
 
  # record some constant settings/values
  for my $vals (qw/protocol TR ET Series nslice/) {
@@ -343,6 +361,35 @@ sub readMRdir {
            $self->{TR}) unless $self->{trustIdx}=~/All|MR/i;
 }
 
+=head2 readBIDSJson
+read TR MRstart and crate MRend from BIDS style json output 
+(e.g. created by dcm2niix). 
+
+must already have nDcms (number of volumes in 4d) set
+
+=cut
+sub readBIDSJson() {
+ use JSON qw/decode_json/;
+ my $self=shift;
+ my $jsonfile=shift;
+ open my $JS, '<', $jsonfile or 
+   croak "could not open BIDS JSON '$jsonfile'";
+ my $data = decode_json(do{local $/; <$JS>})  or
+   croak "could not parse JSON in '$jsonfile'";
+
+ $self->{'MRstart'} = getMRAcqSecs($data->{'AcquisitionDateTime'});
+ # ($+{HH}*60*60) + ($+{MM}*60) + $+{SS} ..
+ $self->{TR} = $data->{RepetitionTime}; # in seconds. eg 1.5
+ $self->{MRend} = $self->{MRstart} + $self->{nDcms}*$self->{TR};
+ # timeCheck is meaningless -- MRend is derived the same way as the check
+           
+ # for output, set protocol and series 
+ $self->{protocol} = $data->{ProtocolName};
+ $self->{Series} = $data->{SeriesDescription};
+ # Also have meaningful attributes
+ # PhaseEncodingDirection SliceTiming
+}
+
 =head2 writeMRPhys
 
 write phys during MR to file
@@ -448,21 +495,22 @@ sub retroTS {
  my $runtype=shift;
  return if $runtype and $runtype =~ /none/i;
 
+ # we need to have both data types
+ croak "need writeMRPhys for both puls and resp" 
+   if ! $self->{dat} || ! -e $self->{dat}->{resp} || ! -e $self->{dat}->{resp};
+
+
  # default to using matlab
  my $matlabbin="matlab";
 
  # find where matlab script points to
- my $acutalmatlab=`perl -ne 'print \$& if /^.*matlab /' \`which matlab\``;
- $matlabbin=$acutalmatlab if $acutalmatlab;
+ #my $acutalmatlab=`perl -ne 'print \$& if /^.*matlab /' \`which matlab\``;
+ #$matlabbin=$acutalmatlab if $acutalmatlab;
 
  # or use env MATLABBIN
  $matlabbin= $ENV{MATLABBIN} if $ENV{MATLABBIN};
 
 
- # we need to have both data types
- croak "need writeMRPhys for both puls and resp" 
-   if ! $self->{dat} || ! -e $self->{dat}->{resp} || ! -e $self->{dat}->{resp};
-
  my %params = (
    "Opts.Respfile"   => "'".$self->{dat}->{resp}."'", # Respiration data file
    "Opts.Cardfile"   => "'".$self->{dat}->{puls}."'", # Cardiac data file
@@ -506,14 +554,25 @@ sub retroTS {
   $runcmd=$mccmd;
  }
 
- system($runcmd) if $runcmd;
 
- if( $runcmd && ! -e "oba.slibase.1D" ){
-   croak "failed to run $runcmd";
- } else {
-   # move file to output name if told to
-   rename "oba.slibase.1D", $outputname if $outputname;
+ if($runcmd) {
+   # retroTS just dumpes out oba.slibase.1D whereever it is run from
+   # ..so run from where we want to save the file
+   chdir dirname($outputname); 
+   system($runcmd);
+
+   # check if we have the expected output
+   if(! -e "oba.slibase.1D" ){
+     croak "failed to run\n\n: $runcmd\n\n";
+   } else {
+     # move file to output name
+     mv "oba.slibase.1D", $outputname or
+        croak "could not move oba.slibase.1D to $outputname";
+   }
+   print "$outputname # saved as final output " , `date +%F\\ %H:%M` , "\n";
  }
+
+
 }
 
 
@@ -545,11 +604,13 @@ sub timeCheck {
  return 1;
 }
 
-# DICOM Acq Time is fmt like HHMMSS.SS
+# DICOM Acq Time is fmt like HHMMSS.SS (172745.487500)
+# JSON BIDs AcqTime like 2018-06-15T17:27:45.487500
 sub getMRAcqSecs {
   $_=shift;
-  m/^(?<HH>\d{2})(?<MM>\d{2})(?<SS>\d{2}\.\d+)$/ 
-    or croak "$_ from MR time does not look like HHMMSS.sssss";
+  m/^(?<HH>\d{2})(?<MM>\d{2})(?<SS>\d{2}\.\d+)$/ or
+   m/^\d{4}-\d{2}-\d{2}T(?<HH>\d{2}):(?<MM>\d{2}):(?<SS>\d{2}\.\d+)$/ or
+   croak "$_ from MR time does not look like HHMMSS.sssss or like YYYY-MM-DDTHH:MM:SS.sssss";
 
   my $secs = ($+{HH}*60*60) + ($+{MM}*60) + $+{SS} ;
   return $secs;
@@ -602,10 +663,12 @@ sub sandwichIdx {
   my ($MRstart,$MRend)     = @{$meat};
   my ($physStart,$physEnd) = @{$bread};
 
-  croak "MR starts ($MRstart) before physio ($physStart)" 
+  croak sprintf("ERROR: MR starts ($MRstart) %.2f min before physio ($physStart)",
+		($physStart-$MRstart)/60)
     if( $MRstart  < $physStart );
 
-  croak "MR ends ($MRend) after physio ($physEnd)" 
+  croak sprintf("ERROR: MR ends ($MRend) %.02f min after physio ($physEnd)",
+		($MRend-$physEnd)/60)
     if( $MRend  > $physEnd);
 
   # calc start and end by adding sandwitch top (start as ref)
